/***************************************************************************

        Auticko - auticko-modeld.c

        copyright (C) 2002 by Josef Kufner <jkufner@seznam.cz>


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

***************************************************************************/

#include <unistd.h>
#include <sys/io.h>
#include <errno.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include "auticko.h"
#include "auticko-modeld.h"

#define OUTPUT_FILE "/tmp/modeld.output.fifo"

#define TOGGLE(x)	x = (x == 0)

unsigned long int modeld_ba = 0x2F8;	/* Port Base Address (/dev/ttyS1) */

static void *Auticko_ModelD_Stop_Thread(void *arg);
static int stop_done = -1;
static pthread_t stop_thread;
static int out_fd;

int Auticko_ModelD_Init(const unsigned long int port) {
	
	out_fd = open(OUTPUT_FILE, O_WRONLY | O_SYNC);
	if (out_fd == -1) {
		return errno;
	}
		
	Auticko_ModelD_Send_Command(CMD_STOP, 0);
	return 0;
}

int Auticko_ModelD_Clean_Up() {
	errno = 0;
	close(out_fd);
	return errno;
}

int Auticko_ModelD_Send_Command(int cmd, int arg) {
	unsigned char byte = 0;

	if (stop_done == 0) {
		stop_done = 1;
		pthread_join(stop_thread, NULL);
	}
	
	switch (cmd) {
		case CMD_STOP:
			stop_done = 0;
			pthread_create(&stop_thread, NULL,
					Auticko_ModelD_Stop_Thread, NULL);
			break;
			
		case CMD_GO:
			byte = (arg != 0);
			write(out_fd, &byte, 1);
			break;
			
		case CMD_LEFT:
			break;
			
		case CMD_RIGHT:
			break;
			
		default:
			return -1;
	}
	return 0;
}


/******** Mistni funkce ******************************************************/

static void *Auticko_ModelD_Stop_Thread(void *arg) {
	unsigned char byte = 0;

	while (!stop_done) {
		byte = 0;
		write(out_fd, &byte, 1);
		usleep(800);
		byte = 1;
		write(out_fd, &byte, 1);
		usleep(1000);
	}
	byte = 0;
	write(out_fd, &byte, 1);

	pthread_exit(NULL);
	return NULL;
}

