/***************************************************************************

        Auticko - auticko-port.c

        copyright (C) 2002 by Josef Kufner <jkufner@seznam.cz>


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

***************************************************************************/

#include <unistd.h>
#include <sys/io.h>
#include <errno.h>
#include <pthread.h>

#include "auticko.h"
#include "auticko-port.h"

#define TOGGLE(x)	x = (x == 0)

unsigned long int ba = 0x3F8;	/* Port Base Address (/dev/ttyS0) */

static void *Auticko_Port_Stop_Thread(void *arg);
static int stop_done = -1;
static pthread_t stop_thread;

int Auticko_Port_Init(const unsigned long int port) {
	
	if (ioperm(ba, 7, 1))
		return errno;
	
	Auticko_Port_Send_Command(CMD_STOP, 0);
	return 0;
}

int Auticko_Port_Clean_Up() {
	if (ioperm(ba, 7, 0))
		return errno;
	return 0;
}

int Auticko_Port_Send_Command(int cmd, int arg) {
	
	if (stop_done == 0) {
		if (cmd == CMD_STOP) {
			return 0;
		}
		
		stop_done = 1;
		pthread_join(stop_thread, NULL);
	}
	
	switch (cmd) {
		case CMD_STOP:
			stop_done = 0;
			pthread_create(&stop_thread, NULL,
					Auticko_Port_Stop_Thread, NULL);
			break;
			
		case CMD_GO:
			outb((arg != 0), ba + 4);	/* DTR = { 0, 1 } */
			break;
			
		case CMD_LEFT:
			break;
			
		case CMD_RIGHT:
			break;
			
		default:
			return -1;
	}
	return 0;
}


/******** Mistni funkce ******************************************************/

static void *Auticko_Port_Stop_Thread(void *arg) {
	while (!stop_done) {
		outb(0, ba + 4);
		usleep(800);
		outb(1, ba + 4);
		usleep(1000);
	}
	outb(0, ba + 4);

	pthread_exit(NULL);
	return NULL;
}

