/***************************************************************************

	Auticko - main.c
	
	copyright (C) 2002 by Josef Kufner <jkufner@seznam.cz>


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
***************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "auticko.h"
#include "ui.h"
#include "config.h"
#include "ui_fn.h"
#include "joystick.h"

#ifdef AUTICKO_MODELD
#define AUTICKO_DRIVER "modeld"
#else
#ifdef AUTICKO_PORT
#define AUTICKO_DRIVER "port"
#endif
#endif

static void on_Exit(void) {
	Auticko_Clean_Up();
	ui_Clean_Up();
}

int main(int argc, char *argv[]) {

	fd_set in_fds;
	int opt, port = 0x3F8, i, in_active;
	
	/* Dekodovani argumentu */
	while((opt = getopt(argc, argv, "p:")) != -1) {
		switch(opt) {
			case 'p':
				sscanf(optarg, "%lx", (unsigned long *)&port);
				break;
		}
	}

	/* Inicializace */
	ui_Init();
	if (Auticko_Init(port) == 0) {
		ui_Terminal("Auticko driver: %s\n", AUTICKO_DRIVER);
	} else {
		ui_Terminal(T_ERR "Auticko: Chyba pi inicializaci (%s):\n\t%s\n",
				AUTICKO_DRIVER, strerror(errno));	
	}
	
	if (joy_Init() == 0) {
		ui_Terminal("Joystick: %d osy, %d tlatka.\n",
				js_state.no_axis, js_state.no_btns);
	} else {
		ui_Terminal(T_ERR "Joystick nenalezen:\n\t%s\n", strerror(errno));
	}
	atexit(on_Exit);


	ui_Terminal(T_OK "Ready.\n");
	ui_Update(main_ui);
	while (1) {
		FD_ZERO(&in_fds);
		FD_SET(0, &in_fds);
		if (js_state.fd > 0)
			FD_SET(js_state.fd, &in_fds);
		
		/* Cekani na vstup */
		if (select(FD_SETSIZE, &in_fds, NULL, NULL, NULL) > 0) {
			/* mame vstup */
			
			/* Ktery? */
			if (FD_ISSET(0, &in_fds))
				in_active = 0;
			if (js_state.fd > 0 && FD_ISSET(js_state.fd, &in_fds))
				in_active = js_state.fd;
				

			/* Reakce */
			if ((i = ui_User(main_ui, in_active)) >= 0)
				(*(main_ui[i].fn))(in_active == js_state.fd ? 0 : main_ui[i].data);
			ui_Update(main_ui);
		}
	}	
	
	
	return 0;
}

