/***************************************************************************

        Auticko - ui.c

        copyright (C) 2002 by Josef Kufner <jkufner@seznam.cz>


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

***************************************************************************/

#include <unistd.h>
#include <ncurses.h>
#include <stdarg.h>
#include <string.h>

#include "ui.h"
#include "joystick.h"

#define LIST_WIDTH	25
#define PICT_HEIGHT	15
#define PICT_WIDTH	30

#define CP_T_ERR	1
#define CP_T_OK		2
#define CP_T_DEBUG	3

#define CP_P		4
#define CP_P_RED	5
#define CP_P_YELLOW	6
#define CP_P_WHITE	7

#define P_HIGHLIGHT	1
#define P_DRAW		2
#define P_BLINK		4
 
static WINDOW *w_key_list = NULL, *w_picture = NULL, *w_terminal = NULL;

struct pict_val_struct pict_val = { 0, 0, 1, 0, 0, 0, 0 };

struct picture_item_struct {
	char *face;		/* obrazek */
	int face_width;		/* sirka obrazku */
	short face_cp;		/* barva */
	int face_style;		/* Styl zobrazeni: { vykreslit, zvyraznit } */
	short x, y;		/* pozice leveho horniho rohu (od 0) */
	int *val;		/* pointer na promennou, kde je stav {0, 1} */
} picture_item[] = {
/* Levy blinkr Pred. */
	{ "o", 1, CP_P_YELLOW, P_BLINK | P_HIGHLIGHT, 
		1, 8, &(pict_val.l_blink) },
/* Pravy blinkr Pred. */
	{ "o", 1, CP_P_YELLOW, P_BLINK | P_HIGHLIGHT,
		1, 20, &(pict_val.r_blink) },
/* Levy blinkr Zad. */
	{ "O", 1, CP_P_YELLOW, P_BLINK | P_HIGHLIGHT,
		13, 6, &(pict_val.l_blink) },
/* Pravy blinkr Zad. */
	{ "O", 1, CP_P_YELLOW, P_BLINK | P_HIGHLIGHT,
		13, 22, &(pict_val.r_blink) },
/* Svetlo Leve predni */
	{ "00", 2, CP_P_WHITE, P_HIGHLIGHT,
		1, 9, &(pict_val.light) },
/* Svetlo Prave predni */
	{ "00", 2, CP_P_WHITE, P_HIGHLIGHT,
		1, 18, &(pict_val.light) },
/* Svetlo Leve zadni */
	{ "O", 1, CP_P_RED, P_HIGHLIGHT,
		13, 8, &(pict_val.light) },
/* Svetlo Prave zadni */
	{ "O", 1, CP_P_RED, P_HIGHLIGHT,
		13, 20, &(pict_val.light) },
/* Sipka vlevo */
	{ "<<", 2, CP_P_RED, P_DRAW,
		4, 12, &(pict_val.left) },
/* Sipka vpravo */
	{ ">>", 2, CP_P_RED, P_DRAW,
		4, 15, &(pict_val.right) },
/* Sipka vpred */
	{ "/^\\/^\\", 3, CP_P_RED, P_DRAW,
		9, 13, &(pict_val.up) },
/* Sipka zpet */
	{ "\\_/\\_/", 3, CP_P_RED, P_DRAW,
		10, 13, &(pict_val.down) },
/* Konec */
	{ NULL, 0, 0, 0, 0, 0, NULL }
};

char *picture_base[] = {		/* Zakladni obrazek */
/*		 0         1         2         3	*/
/*		 0123456789012345678901234567890	*/
/*	 0 */	"      _________________       ",
/*	 1 */	"     ===================      ",
/*	 2 */	"    +-+   $$     $$   +-+     ",
/*	 3 */	"    | |==$$       $$==| |     ",
/*	 4 */	"    +-+ $$         $$ +-+     ",
/*	 5 */	"       $$    /^\\    $$        ",
/*	 6 */	"       $$   |, ,|   $$        ",
/*	 7 */	"       $$   | O |   $$        ",
/*	 8 */	"       $$   #####   $$        ",
/*	 9 */	"   +--+ $$         $$ +--+    ",
/*	10 */	"   |  |==$$       $$==|  |    ",
/*	11 */	"   +--+  $$       $$  +--+    ",
/*	12 */	"    #####################     ",
/*	13 */	"    ^^^^^^^^^^^^^^^^^^^^^     ",
/*	14 */	"                              ",
/* 	15 */	NULL
};

int ui_Init(void) {
	initscr();

	noecho();
	cbreak();
	crmode();
	keypad(stdscr, 1);

	start_color();
	init_pair(CP_T_ERR, COLOR_RED, COLOR_BLACK);
	init_pair(CP_T_OK, COLOR_GREEN, COLOR_BLACK);
	init_pair(CP_T_DEBUG, COLOR_BLUE, COLOR_BLACK);
	init_pair(CP_P, COLOR_BLUE, COLOR_BLACK);
	init_pair(CP_P_RED, COLOR_RED, COLOR_BLACK);
	init_pair(CP_P_YELLOW, COLOR_YELLOW, COLOR_BLACK);
	init_pair(CP_P_WHITE, COLOR_WHITE, COLOR_BLACK);
	
	/* Rozvrzeni oken */
	w_key_list = newwin(LINES, LIST_WIDTH - 2, 0, 1);
	w_picture = newwin(PICT_HEIGHT, PICT_WIDTH,
			1,  LIST_WIDTH + 2 + 
			((COLS - (LIST_WIDTH + 4 + PICT_WIDTH)) / 2));
	w_terminal = newwin(LINES - PICT_HEIGHT - 3,
			COLS - LIST_WIDTH - 3,
			PICT_HEIGHT + 3,
			LIST_WIDTH + 2);
	scrollok(w_terminal, 1);

	ui_Refresh(NULL);
	return 0;
}

int ui_Clean_Up (void) {
	endwin();
	return 0;
}

int ui_User(const UI_FN *ui_fn, int input) {
	int key = 0, i;

	if (input == fileno(stdin))
		key = getch();	
	else
		key = joy_Get_Event();
	
	for (i = 0; ui_fn != NULL && ui_fn[i].title != NULL; i++) {
		if ((ui_fn[i].key == key || ui_fn[i].joystick == key) && ui_fn[i].fn != NULL) {
			return i;
		}	
	}
	
	return -1;
}

int ui_Refresh(const UI_FN *ui_fn) {

	int i = 0;
	
	
	/* Vymaz obrazovky */
	erase();


	/* Vykresleni ramecku */
	for (i = 0; i < LINES; i++)
		mvaddch(i, LIST_WIDTH, ACS_VLINE);
	
	for (i = LIST_WIDTH + 1; i < COLS; i++)
		mvaddch(PICT_HEIGHT + 2, i, ACS_HLINE);
	
	wnoutrefresh(w_key_list);
	wnoutrefresh(w_picture);
	wnoutrefresh(w_terminal);

	ui_Update(ui_fn);
	
	return 0;
}

int ui_Update(const UI_FN *ui_fn) {	
	int i;

	/* Aktualizace seznamu klaves (nabidka) */
	wattron(w_key_list, A_BOLD);
	mvwprintw(w_key_list, 0, 2, "Menu:");
	wattroff(w_key_list, A_BOLD);
	
	for (i = 0; ui_fn != NULL && ui_fn[i].title != NULL; i++) {
		if (strlen(ui_fn[i].keyname) != 0) {
			mvwprintw(w_key_list, i + 2,
					LIST_WIDTH
					- strlen(ui_fn[i].keyname) - 4,
					"<%s>", ui_fn[i].keyname);
		}
		mvwprintw(w_key_list, i + 2, 0, "%s", ui_fn[i].title);
	}
	wnoutrefresh(w_key_list);
	
	doupdate();
	refresh();
	ui_Draw_Picture();
	wrefresh(w_terminal);
	return 0;
}

int ui_Question(const char *str, const unsigned short btn, const int style) {
	return 0;
}

int ui_StrIn(const char *label, char *buf, const int maxlen) {
	return 0;
}

void ui_Terminal(char *format, ...) {
	va_list arg;
	char str[1024];

	switch (format[0]) {
		case T_ERR_I:
			wattron(w_terminal, COLOR_PAIR(CP_T_ERR) | A_BOLD);
			format++;
			break;
			
		case T_OK_I:
			wattron(w_terminal, COLOR_PAIR(CP_T_OK) | A_BOLD);
			format++;
			break;
			
		case T_DEBUG_I:
			wattron(w_terminal, COLOR_PAIR(CP_T_DEBUG));
			format++;
			break;
			
		default:
			break;
	}
	
	va_start(arg, format);
	vsprintf(str, format, arg);
	va_end(arg);
	
	wprintw(w_terminal, str);

	switch (format[-1]) {
		case T_ERR_I:
			wattroff(w_terminal, COLOR_PAIR(CP_T_ERR) | A_BOLD);
			break;
		
		case T_OK_I:
			wattroff(w_terminal, COLOR_PAIR(CP_T_OK) | A_BOLD);
			break;
		
		case T_DEBUG_I:
			wattroff(w_terminal, COLOR_PAIR(CP_T_DEBUG));
			format++;
			break;
			
		default:
			break;
	}
	
	wrefresh(w_terminal);
}

void ui_Draw_Picture (void) {
	int i, ch;
	chtype attr;
	
	wattron(w_picture, COLOR_PAIR(CP_P) | A_DIM);
	for (i = 0; picture_base[i] != NULL; i++) {
		mvwprintw(w_picture, i, 0, "%s", picture_base[i]);
	}
	wattroff(w_picture, COLOR_PAIR(CP_P) | A_DIM);

	for (i = 0; picture_item[i].face != NULL; i++) {

		/* Nastaveni atributu */		
		attr = COLOR_PAIR(picture_item[i].face_cp);
		if (*(picture_item[i].val) != 0) {
			if (picture_item[i].face_style & P_HIGHLIGHT)
				attr |= A_BOLD;
			if (picture_item[i].face_style & P_BLINK)
				attr |= A_BLINK;
		} else {
			if (picture_item[i].face_style & P_DRAW)
				continue;
		}
		wattron(w_picture, attr);
		
		/* Prekresleni objektu */
		for (ch = 0; picture_item[i].face[ch] != 0; ch++) {
			mvwaddch(w_picture, 
					picture_item[i].x + (ch 
						/ picture_item[i].face_width), 
					picture_item[i].y + (ch
						% picture_item[i].face_width),
					picture_item[i].face[ch]);
		}

		/* Vypnuti atributu */
		wattroff(w_picture, attr);
	}
	
	wrefresh(w_picture);
}

